#ifndef CONNECT4_H
#define CONNECT4_H
#include <string>
#include <vector>
#include "piece.h"
#include "mesh.h"

enum {
	FF_TRIANGLE,
	FF_SQUARE,
	FF_CIRCLE,
	FF_CROSS,
	FF_LEFT,
	FF_UP,
	FF_RIGHT,
	FF_DOWN,
	FF_LTRIGGER,
	FF_RTRIGGER,
	FF_SELECT,
	FF_START,
	FF_HOLD,
	FF_MAX		// Last one.
};

enum {
S_INTRO,
S_LEVELSTART,
S_WINNER,
S_COMPUTERWINNER,
S_DROPPIECE,
S_BADMOVE,
S_MOVEARROW,
S_MAX
};

/** To be implemented by game engine. Plays an appropriate sound.
 * \param id
 */
void playSound(int id);

enum BoardPiece {
	BP_EMPTY,
	BP_RED,
	BP_YELLOW,
	BP_REDPROPOSED,	// used by the AI to search
	BP_YELLOWPROPOSED,	// used by the AI to search
	BP_TIE	// used by the AI to search
};

enum GameMode {
	GM_TITLE,
	GM_MODEMENU,
	GM_PVSP,	// player versus player
	GM_PVSAI,	// player versus ai
	GM_SERVER,
	GM_CLIENT,
	GM_SERVERWAITING,
	GM_CLIENTWAITING,
	GM_REDWINS,
	GM_YELLOWWINS,
	GM_CPUWINS,
	GM_TIE,
	GM_QUIT
};

class Entity;
/**
 * This is the main game, which is event/timer driven.
 */
class Connect4
{
	public:
		// class constructor
		Connect4();
		// class destructor
		~Connect4();
		Entity * getSceneEntity(); // returns the value of sceneEntity
		int getScore(); // returns the value of score
		void switchTheme();
		/**
		 * New game is called to initialize the game engine.  It resets
		 * the score, health, level, loads the initial level, and resets 
		 * the NPCs.
		 */
		void newGame();
		/**
		 * Updates the state of all entities in the currently active level.
		 * 
		 * @param milliseconds milliseconds since the game/level/machine started.
		 */
		void updateLevel(unsigned long milliseconds);
		std::vector<Piece *> piece;
		/**
		 * Loads in the level specified, and initializes all things needed.
		 * 
		 * @param level the level number to load (0 based)
		 * @returns 0 on success, negative on error.
		 */
		int load(int level);
		/**
		 * A keyboard event was received.
		 * 
		 * @param ch ASCII character or special keycode.
		 * @param down true if pressed, or false if released.
		 */
		void keyEvent(int ch,int down);
		/**
		 * Receives a button event.
		 * 
		 * @param id the button id of the button (0=triangle, etc).
		 * @param down true if the button is pressed, and false if the button was released.
		 */
		void buttonEvent(int id,int down,int milliseconds);
		/**
		 * The scene is static meshes for the level which is currently loaded.
		 */
		Entity * sceneEntity;

		/** if we should be showing the pause menu */
		bool pauseMenu;
		/** sound is enabled */
		bool sound;
		/** Mode of the game engine */
		enum GameMode mode;
		/** arrow column */
		int getArrowColumn() { return arrowColumn; }
		/** Which player's turn it is */
		enum BoardPiece activePlayer;
	protected:
        /** list of the themes. */
        std::vector<std::string> theme;
        /** the active theme index */
        unsigned int themeNo;
		/** rotate the piece around the right way.
		 */
		void orientPiece(Entity *modelPiece);
		/** calculate the board relative position of a piece.
		 * \param column in bp squares
		 * \param row in bp squares
		 * \param x output of calc
		 * \param y output of calc
		 */
		void calcPiecePos(int row,int column,float &x,float &y,float &z);
		/** check a cell with bounds checking.
		 * \param player the player to check for (player/proposed/empty)
		 * \param x the column
		 * \param y the row
		 * \returns 0 if non player, 1 if empty, 5 if filled.
		 */
		int isPlayer(enum BoardPiece player,int x,int y);
		/** find an empty row for the next token.
		 * \returns -1 if row is full, or the row from 0 to 5
		 */
		int getRow(int column);
		/** drops a piece onto the map as appropriate
		 * \param arrowColumn the column to add the piece to.
		 */
		void dropPiece(int arrowColumn);
		/** Search for move 
		 * \param player either BP_RED or BP_YELLOW
		 * \returns column to move in (0 to 6).
		 */
		int searchForMove(enum BoardPiece player);
		/** Search for move 
		 * \param player either BP_RED or BP_YELLOW
		 * \param alpha goodness of a particular move
		 * \param beta badness of a particular move
		 * \returns column to move in (0 to 6).
		 */
		int searchForMove(enum BoardPiece player,int maxDepth,int alpha,int beta,int &bestColumn);
		/** Measure the combinations for a given direction
		 * \param player the current player
		 * \param x the column
		 * \param y the row
		 * \param dx direction x (-1, 0, 1)
		 * \param dy direction y (-1, 0, 1)
		 * \returns a value from 0 to 1*4=4 to (5*3+1)*4=64
		 */
		int potientialFour(enum BoardPiece player,int x,int y,int dx,int dy);
		/**
		 * a heuristic for the worth of the board position.
		 */
		int positionPotiential(enum BoardPiece player);
		/** Search for a winning combination
		 * \returns BP_NONE, BP_RED, BP_YELLOW, BP_REDPROPOSED, BP_YELLOWPROPOSED
		 */
		enum BoardPiece winCondition();
		/** clears out the proposed tiles */
		void clearProposals();
		/** Model of board */
		Entity *modelBoard;
		/** Model of red piece */
		Entity *modelRed;
		/** Model of yellow piece */
		Entity *modelYellow;
		/** Model of arrow */
		Entity *modelArrow;
		/** Model of room */
		Entity *modelRoom;
		/** displayable board */
		enum BoardPiece bp[6][7];
		/** Arrow pointer. */
		int arrowColumn;
		/** Arrow for HUD. */
		Piece *arrowPiece;
		/** Token for HUD.
		 * Displays the current player's token, on the HUD.
		 */
		Piece *tokenPiece;	

		/**
		 * The current displayed score.  This is updated by this class, 
		 * and  displayed in the HUD.
		 */
		int score;
		/**
		 * Last parameter to updateLevel.
		 */
		unsigned long lastTick;
		/**
		 * This is the map of what buttons are down, and what ones are up.
		 * This determines what actions to take to update the hero position.
		 */
		int buttonState;
	protected:
};

#endif
